@@ -1130,7 +1130,7 @@ route_set_community (void *rule, struct prefix *prefix,
   struct community *new = NULL;
   struct community *old;
   struct community *merge;
-
+  
   if (type == RMAP_BGP)
     {
       rcs = rule;
@@ -1150,12 +1150,20 @@ route_set_community (void *rule, struct prefix *prefix,
       if (rcs->additive && old)
 	{
 	  merge = community_merge (community_dup (old), rcs->com);
+	  
+	  /* HACK: if the old community is not intern'd, 
+           * we should free it here, or all reference to it may be lost.
+           * Really need to cleanup attribute caching sometime.
+           */
+	  if (old->refcnt == 0)
+	    community_free (old);
 	  new = community_uniq_sort (merge);
 	  community_free (merge);
 	}
       else
 	new = community_dup (rcs->com);
-
+      
+      /* will be interned by caller if required */
       attr->community = new;
 
       attr->flag |= ATTR_FLAG_BIT (BGP_ATTR_COMMUNITIES);
