@@ -199,7 +199,10 @@ prefix_ipv4_new ()
 {
   struct prefix_ipv4 *p;
 
-  p = XCALLOC (MTYPE_PREFIX_IPV4, sizeof *p);
+  /* Call prefix_new to allocate a full-size struct prefix to avoid problems
+     where the struct prefix_ipv4 is cast to struct prefix and unallocated
+     bytes were being referenced (e.g. in structure assignments). */
+  p = (struct prefix_ipv4 *)prefix_new();
   p->family = AF_INET;
   return p;
 }
@@ -208,7 +211,7 @@ prefix_ipv4_new ()
 void
 prefix_ipv4_free (struct prefix_ipv4 *p)
 {
-  XFREE (MTYPE_PREFIX_IPV4, p);
+  prefix_free((struct prefix *)p);
 }
 
 /* When string format is invalid return 0. */
@@ -292,7 +295,7 @@ ip_masklen (struct in_addr netmask)
   pnt = (u_char *) &netmask;
   end = pnt + 4;
 
-  while ((*pnt == 0xff) && pnt < end)
+  while ((pnt < end) && (*pnt == 0xff))
     {
       len+= 8;
       pnt++;
@@ -348,7 +351,9 @@ prefix_ipv6_new ()
 {
   struct prefix_ipv6 *p;
 
-  p = XCALLOC (MTYPE_PREFIX_IPV6, sizeof (struct prefix_ipv6));
+  /* Allocate a full-size struct prefix to avoid problems with structure
+     size mismatches. */
+  p = (struct prefix_ipv6 *)prefix_new();
   p->family = AF_INET6;
   return p;
 }
@@ -357,7 +362,7 @@ prefix_ipv6_new ()
 void
 prefix_ipv6_free (struct prefix_ipv6 *p)
 {
-  XFREE (MTYPE_PREFIX_IPV6, p);
+  prefix_free((struct prefix *)p);
 }
 
 /* If given string is valid return pin6 else return NULL */
@@ -735,3 +740,14 @@ netmask_str2prefix_str (const char *net_str, const char *mask_str,
   return 1;
 }
 
+#ifdef HAVE_IPV6
+/* Utility function for making IPv6 address string. */
+const char *
+inet6_ntoa (struct in6_addr addr)
+{
+  static char buf[INET6_ADDRSTRLEN];
+
+  inet_ntop (AF_INET6, &addr, buf, INET6_ADDRSTRLEN);
+  return buf;
+}
+#endif /* HAVE_IPV6 */
