@@ -1851,7 +1851,7 @@ ospf6_init ()
   install_element (ENABLE_NODE, &show_ipv6_ospf6_linkstate_detail_cmd);
 
 #define INSTALL(n,c) \
-  install_element (n ## _NODE, &show_ipv6_ospf6_ ## c);
+  install_element (n ## _NODE, &show_ipv6_ospf6_ ## c)
 
   INSTALL (VIEW, database_cmd);
   INSTALL (VIEW, database_detail_cmd);
@@ -1889,8 +1889,6 @@ ospf6_init ()
   INSTALL (VIEW, database_type_id_self_originated_detail_cmd);
   INSTALL (VIEW, database_type_self_originated_linkstate_id_cmd);
   INSTALL (VIEW, database_type_self_originated_linkstate_id_detail_cmd);
-  INSTALL (VIEW, database_type_id_self_originated_cmd);
-  INSTALL (VIEW, database_type_id_self_originated_detail_cmd);
 
   INSTALL (ENABLE, database_cmd);
   INSTALL (ENABLE, database_detail_cmd);
@@ -1928,8 +1926,6 @@ ospf6_init ()
   INSTALL (ENABLE, database_type_id_self_originated_detail_cmd);
   INSTALL (ENABLE, database_type_self_originated_linkstate_id_cmd);
   INSTALL (ENABLE, database_type_self_originated_linkstate_id_detail_cmd);
-  INSTALL (ENABLE, database_type_id_self_originated_cmd);
-  INSTALL (ENABLE, database_type_id_self_originated_detail_cmd);
 
   /* Make ospf protocol socket. */
   ospf6_serv_sock ();
