@@ -237,9 +237,6 @@ ospf_if_new (struct ospf *ospf, struct interface *ifp, struct prefix *p)
 
   /* Add pseudo neighbor. */
   oi->nbr_self = ospf_nbr_new (oi);
-  oi->nbr_self->state = NSM_TwoWay;
-  oi->nbr_self->priority = OSPF_IF_PARAM (oi, priority);
-  oi->nbr_self->options = OSPF_OPTION_E;
 
   oi->ls_upd_queue = route_table_init ();
   oi->t_ls_upd_event = NULL;
@@ -265,7 +262,7 @@ ospf_if_cleanup (struct ospf_interface *oi)
   struct listnode *node;
   struct ospf_neighbor *nbr;
 
-  /* oi->nbrs and oi->nbr_nbma should be deletete on InterafceDown event */
+  /* oi->nbrs and oi->nbr_nbma should be deleted on InterfaceDown event */
   /* delete all static neighbors attached to this interface */
   for (node = listhead (oi->nbr_nbma); node; )
     {
@@ -301,26 +298,11 @@ ospf_if_cleanup (struct ospf_interface *oi)
   /* Empty link state update queue */
   ospf_ls_upd_queue_empty (oi);
   
- /* Handle pseudo neighbor. */
+  /* Reset pseudo neighbor. */
   ospf_nbr_delete (oi->nbr_self);
   oi->nbr_self = ospf_nbr_new (oi);
-  oi->nbr_self->state = NSM_TwoWay;
-  oi->nbr_self->priority = OSPF_IF_PARAM (oi, priority);
-
-  switch (oi->area->external_routing)
-    {
-    case OSPF_AREA_DEFAULT:
-      SET_FLAG (oi->nbr_self->options, OSPF_OPTION_E);
-      break;
-    case OSPF_AREA_STUB:
-      UNSET_FLAG (oi->nbr_self->options, OSPF_OPTION_E);
-      break;
-    case OSPF_AREA_NSSA:
-      UNSET_FLAG (oi->nbr_self->options, OSPF_OPTION_E);
-      SET_FLAG (oi->nbr_self->options, OSPF_OPTION_NP);
-      break;
-    }
-
+  ospf_nbr_add_self (oi);
+  
   ospf_lsa_unlock (oi->network_lsa_self);
   oi->network_lsa_self = NULL;
   OSPF_TIMER_OFF (oi->t_network_lsa_self);
@@ -496,6 +478,9 @@ ospf_if_lookup_recv_if (struct ospf *ospf, struct in_addr src)
       if (oi->type == OSPF_IFTYPE_VIRTUALLINK)
 	continue;
       
+      if (if_is_loopback (oi->ifp))
+        continue;
+      
       if ((oi->type == OSPF_IFTYPE_POINTOPOINT) &&
 	  CONNECTED_DEST_HOST(oi->connected))
 	{
@@ -873,6 +858,7 @@ ospf_vl_new (struct ospf *ospf, struct ospf_vl_data *vl_data)
   if (IS_DEBUG_OSPF_EVENT)
     zlog_debug ("ospf_vl_new(): set associated area to the backbone");
 
+  ospf_nbr_add_self (voi);
   ospf_area_add_if (voi->area, voi);
 
   ospf_if_stream_set (voi);
