@@ -73,7 +73,7 @@ const char *nexthop_types_desc[] =
   "IPv4 nexthop",
   "IPv4 nexthop with ifindex",
   "IPv4 nexthop with ifname",
-  "IPv6 nexthop"
+  "IPv6 nexthop",
   "IPv6 nexthop with ifindex",
   "IPv6 nexthop with ifname",
   "Null0 nexthop",
@@ -364,7 +364,7 @@ netlink_parse_info (int (*filter) (struct sockaddr_nl *, struct nlmsghdr *),
                 {
                   if (IS_ZEBRA_DEBUG_KERNEL)
                     {
-                      zlog_debug ("%s: %s ACK: type=%s(%u), seq=%u, pid=%d",
+                      zlog_debug ("%s: %s ACK: type=%s(%u), seq=%u, pid=%u",
                                  __FUNCTION__, nl->name,
                                  lookup (nlmsg_str, err->msg.nlmsg_type),
                                  err->msg.nlmsg_type, err->msg.nlmsg_seq,
@@ -398,7 +398,7 @@ netlink_parse_info (int (*filter) (struct sockaddr_nl *, struct nlmsghdr *),
                   loglvl = LOG_DEBUG;
 
                 zlog (NULL, loglvl, "%s error: %s, type=%s(%u), "
-                      "seq=%u, pid=%d",
+                      "seq=%u, pid=%u",
                       nl->name, safe_strerror (-errnum),
                       lookup (nlmsg_str, msg_type),
                       msg_type, err->msg.nlmsg_seq, err->msg.nlmsg_pid);
@@ -412,7 +412,7 @@ netlink_parse_info (int (*filter) (struct sockaddr_nl *, struct nlmsghdr *),
 
           /* OK we got netlink message. */
           if (IS_ZEBRA_DEBUG_KERNEL)
-            zlog_debug ("netlink_parse_info: %s type %s(%u), seq=%u, pid=%d",
+            zlog_debug ("netlink_parse_info: %s type %s(%u), seq=%u, pid=%u",
                        nl->name,
                        lookup (nlmsg_str, h->nlmsg_type), h->nlmsg_type,
                        h->nlmsg_seq, h->nlmsg_pid);
@@ -422,7 +422,7 @@ netlink_parse_info (int (*filter) (struct sockaddr_nl *, struct nlmsghdr *),
             {
               if (IS_ZEBRA_DEBUG_KERNEL)
                 zlog_debug ("netlink_parse_info: %s packet comes from %s",
-                           nl->name, netlink_cmd.name);
+                            netlink_cmd.name, nl->name);
               continue;
             }
 
@@ -574,23 +574,20 @@ netlink_interface_addr (struct sockaddr_nl *snl, struct nlmsghdr *h)
   if (IS_ZEBRA_DEBUG_KERNEL)    /* remove this line to see initial ifcfg */
     {
       char buf[BUFSIZ];
-      zlog_debug ("netlink_interface_addr %s %s/%d:",
-                 lookup (nlmsg_str, h->nlmsg_type),
-                 ifp->name, ifa->ifa_prefixlen);
+      zlog_debug ("netlink_interface_addr %s %s:",
+                 lookup (nlmsg_str, h->nlmsg_type), ifp->name);
       if (tb[IFA_LOCAL])
-        zlog_debug ("  IFA_LOCAL     %s", inet_ntop (ifa->ifa_family,
-                                                    RTA_DATA (tb[IFA_LOCAL]),
-                                                    buf, BUFSIZ));
+        zlog_debug ("  IFA_LOCAL     %s/%d",
+		    inet_ntop (ifa->ifa_family, RTA_DATA (tb[IFA_LOCAL]),
+			       buf, BUFSIZ), ifa->ifa_prefixlen);
       if (tb[IFA_ADDRESS])
-        zlog_debug ("  IFA_ADDRESS   %s", inet_ntop (ifa->ifa_family,
-                                                    RTA_DATA (tb
-                                                              [IFA_ADDRESS]),
-                                                    buf, BUFSIZ));
+        zlog_debug ("  IFA_ADDRESS   %s/%d",
+		    inet_ntop (ifa->ifa_family, RTA_DATA (tb[IFA_ADDRESS]),
+                               buf, BUFSIZ), ifa->ifa_prefixlen);
       if (tb[IFA_BROADCAST])
-        zlog_debug ("  IFA_BROADCAST %s", inet_ntop (ifa->ifa_family,
-                                                    RTA_DATA (tb
-                                                              [IFA_BROADCAST]),
-                                                    buf, BUFSIZ));
+        zlog_debug ("  IFA_BROADCAST %s/%d",
+		    inet_ntop (ifa->ifa_family, RTA_DATA (tb[IFA_BROADCAST]),
+			       buf, BUFSIZ), ifa->ifa_prefixlen);
       if (tb[IFA_LABEL] && strcmp (ifp->name, RTA_DATA (tb[IFA_LABEL])))
         zlog_debug ("  IFA_LABEL     %s", (char *)RTA_DATA (tb[IFA_LABEL]));
     }
@@ -760,7 +757,8 @@ netlink_routing_table (struct sockaddr_nl *snl, struct nlmsghdr *h)
       memcpy (&p.prefix, dest, 16);
       p.prefixlen = rtm->rtm_dst_len;
 
-      rib_add_ipv6 (ZEBRA_ROUTE_KERNEL, flags, &p, gate, index, table);
+      rib_add_ipv6 (ZEBRA_ROUTE_KERNEL, flags, &p, gate, index, table,
+		    metric, 0);
     }
 #endif /* HAVE_IPV6 */
 
@@ -910,7 +908,7 @@ netlink_route_change (struct sockaddr_nl *snl, struct nlmsghdr *h)
         }
 
       if (h->nlmsg_type == RTM_NEWROUTE)
-        rib_add_ipv6 (ZEBRA_ROUTE_KERNEL, 0, &p, gate, index, 0);
+        rib_add_ipv6 (ZEBRA_ROUTE_KERNEL, 0, &p, gate, index, 0, 0, 0);
       else
         rib_delete_ipv6 (ZEBRA_ROUTE_KERNEL, 0, &p, gate, index, 0);
     }
@@ -1428,62 +1426,114 @@ netlink_route_multipath (int cmd, struct prefix *p, struct rib *rib,
                     {
                       zlog_debug
                         ("netlink_route_multipath() (recursive, 1 hop): "
-                         "%s %s/%d via %s if %u, type %s",
-                         lookup (nlmsg_str, cmd), inet_ntoa (p->u.prefix4),
-                         p->prefixlen, inet_ntoa (nexthop->rgate.ipv4),
-                         nexthop->rifindex,
-                         nexthop_types_desc[nexthop->rtype]);
+                         "%s %s/%d, type %s", lookup (nlmsg_str, cmd),
+#ifdef HAVE_IPV6
+			 (family == AF_INET) ? inet_ntoa (p->u.prefix4) :
+			 inet6_ntoa (p->u.prefix6),
+#else
+			 inet_ntoa (p->u.prefix4),
+#endif /* HAVE_IPV6 */
+			 
+			 p->prefixlen, nexthop_types_desc[nexthop->rtype]);
                     }
 
                   if (nexthop->rtype == NEXTHOP_TYPE_IPV4
                       || nexthop->rtype == NEXTHOP_TYPE_IPV4_IFINDEX)
-                    addattr_l (&req.n, sizeof req, RTA_GATEWAY,
-                               &nexthop->rgate.ipv4, bytelen);
+		    {
+		      addattr_l (&req.n, sizeof req, RTA_GATEWAY,
+				 &nexthop->rgate.ipv4, bytelen);
+
+		      if (IS_ZEBRA_DEBUG_KERNEL)
+			zlog_debug("netlink_route_multipath() (recursive, "
+				   "1 hop): nexthop via %s if %u",
+				   inet_ntoa (nexthop->rgate.ipv4),
+				   nexthop->rifindex);
+		    }
 #ifdef HAVE_IPV6
                   if (nexthop->rtype == NEXTHOP_TYPE_IPV6
                       || nexthop->rtype == NEXTHOP_TYPE_IPV6_IFINDEX
                       || nexthop->rtype == NEXTHOP_TYPE_IPV6_IFNAME)
-                    addattr_l (&req.n, sizeof req, RTA_GATEWAY,
-                               &nexthop->rgate.ipv6, bytelen);
+		    {
+		      addattr_l (&req.n, sizeof req, RTA_GATEWAY,
+				 &nexthop->rgate.ipv6, bytelen);
+
+		      if (IS_ZEBRA_DEBUG_KERNEL)
+			zlog_debug("netlink_route_multipath() (recursive, "
+				   "1 hop): nexthop via %s if %u",
+				   inet6_ntoa (nexthop->rgate.ipv6),
+				   nexthop->rifindex);
+		    }
 #endif /* HAVE_IPV6 */
                   if (nexthop->rtype == NEXTHOP_TYPE_IFINDEX
                       || nexthop->rtype == NEXTHOP_TYPE_IFNAME
                       || nexthop->rtype == NEXTHOP_TYPE_IPV4_IFINDEX
                       || nexthop->rtype == NEXTHOP_TYPE_IPV6_IFINDEX
                       || nexthop->rtype == NEXTHOP_TYPE_IPV6_IFNAME)
-                    addattr32 (&req.n, sizeof req, RTA_OIF,
-                               nexthop->rifindex);
+		    {
+		      addattr32 (&req.n, sizeof req, RTA_OIF,
+				 nexthop->rifindex);
+
+		      if (IS_ZEBRA_DEBUG_KERNEL)
+			zlog_debug("netlink_route_multipath() (recursive, "
+				   "1 hop): nexthop via if %u",
+				   nexthop->rifindex);
+		    }
                 }
               else
                 {
                   if (IS_ZEBRA_DEBUG_KERNEL)
                     {
                       zlog_debug
-                        ("netlink_route_multipath(): (single hop)"
-                         "%s %s/%d via %s if %u, type %s",
-                         lookup (nlmsg_str, cmd), inet_ntoa (p->u.prefix4),
-                         p->prefixlen, inet_ntoa (nexthop->gate.ipv4),
-                         nexthop->ifindex,
-                         nexthop_types_desc[nexthop->type]);
+                        ("netlink_route_multipath() (single hop): "
+                         "%s %s/%d, type %s", lookup (nlmsg_str, cmd),
+#ifdef HAVE_IPV6
+			 (family == AF_INET) ? inet_ntoa (p->u.prefix4) :
+			 inet6_ntoa (p->u.prefix6),
+#else
+			 inet_ntoa (p->u.prefix4),
+#endif /* HAVE_IPV6 */
+			 p->prefixlen, nexthop_types_desc[nexthop->type]);
                     }
 
                   if (nexthop->type == NEXTHOP_TYPE_IPV4
                       || nexthop->type == NEXTHOP_TYPE_IPV4_IFINDEX)
-                    addattr_l (&req.n, sizeof req, RTA_GATEWAY,
-                               &nexthop->gate.ipv4, bytelen);
+		    {
+		      addattr_l (&req.n, sizeof req, RTA_GATEWAY,
+				 &nexthop->gate.ipv4, bytelen);
+
+		      if (IS_ZEBRA_DEBUG_KERNEL)
+			zlog_debug("netlink_route_multipath() (single hop): "
+				   "nexthop via %s if %u",
+				   inet_ntoa (nexthop->gate.ipv4),
+				   nexthop->ifindex);
+		    }
 #ifdef HAVE_IPV6
                   if (nexthop->type == NEXTHOP_TYPE_IPV6
                       || nexthop->type == NEXTHOP_TYPE_IPV6_IFNAME
                       || nexthop->type == NEXTHOP_TYPE_IPV6_IFINDEX)
-                    addattr_l (&req.n, sizeof req, RTA_GATEWAY,
-                               &nexthop->gate.ipv6, bytelen);
+		    {
+		      addattr_l (&req.n, sizeof req, RTA_GATEWAY,
+				 &nexthop->gate.ipv6, bytelen);
+
+		      if (IS_ZEBRA_DEBUG_KERNEL)
+			zlog_debug("netlink_route_multipath() (single hop): "
+				   "nexthop via %s if %u",
+				   inet6_ntoa (nexthop->gate.ipv6),
+				   nexthop->ifindex);
+		    }
 #endif /* HAVE_IPV6 */
                   if (nexthop->type == NEXTHOP_TYPE_IFINDEX
                       || nexthop->type == NEXTHOP_TYPE_IFNAME
                       || nexthop->type == NEXTHOP_TYPE_IPV4_IFINDEX
                       || nexthop->type == NEXTHOP_TYPE_IPV6_IFINDEX
                       || nexthop->type == NEXTHOP_TYPE_IPV6_IFNAME)
-                    addattr32 (&req.n, sizeof req, RTA_OIF, nexthop->ifindex);
+		    {
+		      addattr32 (&req.n, sizeof req, RTA_OIF, nexthop->ifindex);
+
+		      if (IS_ZEBRA_DEBUG_KERNEL)
+			zlog_debug("netlink_route_multipath() (single hop): "
+				   "nexthop via if %u", nexthop->ifindex);
+		    }
                 }
 
               if (cmd == RTM_NEWROUTE)
@@ -1526,12 +1576,15 @@ netlink_route_multipath (int cmd, struct prefix *p, struct rib *rib,
                   if (IS_ZEBRA_DEBUG_KERNEL)
                     {
                       zlog_debug ("netlink_route_multipath() "
-                         "(recursive, multihop): "
-                         "%s %s/%d via %s if %u, type %s",
-                         lookup (nlmsg_str, cmd), inet_ntoa (p->u.prefix4),
-                         p->prefixlen, inet_ntoa (nexthop->rgate.ipv4),
-                         nexthop->rifindex, 
-                         nexthop_types_desc[nexthop->type]);
+                         "(recursive, multihop): %s %s/%d type %s",
+			 lookup (nlmsg_str, cmd),
+#ifdef HAVE_IPV6
+			 (family == AF_INET) ? inet_ntoa (p->u.prefix4) :
+			 inet6_ntoa (p->u.prefix6),
+#else
+			 inet_ntoa (p->u.prefix4),
+#endif /* HAVE_IPV6 */
+                         p->prefixlen, nexthop_types_desc[nexthop->rtype]);
                     }
                   if (nexthop->rtype == NEXTHOP_TYPE_IPV4
                       || nexthop->rtype == NEXTHOP_TYPE_IPV4_IFINDEX)
@@ -1539,13 +1592,27 @@ netlink_route_multipath (int cmd, struct prefix *p, struct rib *rib,
                       rta_addattr_l (rta, 4096, RTA_GATEWAY,
                                      &nexthop->rgate.ipv4, bytelen);
                       rtnh->rtnh_len += sizeof (struct rtattr) + 4;
+
+		      if (IS_ZEBRA_DEBUG_KERNEL)
+			zlog_debug("netlink_route_multipath() (recursive, "
+				   "multihop): nexthop via %s if %u",
+				   inet_ntoa (nexthop->rgate.ipv4),
+				   nexthop->rifindex);
                     }
 #ifdef HAVE_IPV6
                   if (nexthop->rtype == NEXTHOP_TYPE_IPV6
                       || nexthop->rtype == NEXTHOP_TYPE_IPV6_IFNAME
                       || nexthop->rtype == NEXTHOP_TYPE_IPV6_IFINDEX)
-                    rta_addattr_l (rta, 4096, RTA_GATEWAY,
-                                   &nexthop->rgate.ipv6, bytelen);
+		    {
+		      rta_addattr_l (rta, 4096, RTA_GATEWAY,
+				     &nexthop->rgate.ipv6, bytelen);
+
+		      if (IS_ZEBRA_DEBUG_KERNEL)
+			zlog_debug("netlink_route_multipath() (recursive, "
+				   "multihop): nexthop via %s if %u",
+				   inet6_ntoa (nexthop->rgate.ipv6),
+				   nexthop->rifindex);
+		    }
 #endif /* HAVE_IPV6 */
                   /* ifindex */
                   if (nexthop->rtype == NEXTHOP_TYPE_IFINDEX
@@ -1553,35 +1620,60 @@ netlink_route_multipath (int cmd, struct prefix *p, struct rib *rib,
                       || nexthop->rtype == NEXTHOP_TYPE_IPV4_IFINDEX
                       || nexthop->rtype == NEXTHOP_TYPE_IPV6_IFINDEX
                       || nexthop->rtype == NEXTHOP_TYPE_IPV6_IFNAME)
-                    rtnh->rtnh_ifindex = nexthop->rifindex;
+		    {
+		      rtnh->rtnh_ifindex = nexthop->rifindex;
+
+		      if (IS_ZEBRA_DEBUG_KERNEL)
+			zlog_debug("netlink_route_multipath() (recursive, "
+				   "multihop): nexthop via if %u",
+				   nexthop->rifindex);
+		    }
                   else
-                    rtnh->rtnh_ifindex = 0;
+		    {
+		      rtnh->rtnh_ifindex = 0;
+		    }
                 }
               else
                 {
                   if (IS_ZEBRA_DEBUG_KERNEL)
                     {
-                      zlog_debug ("netlink_route_multipath() "
-                         "(multihop): "
-                         "%s %s/%d via %s if %u, type %s",
-                         lookup (nlmsg_str, cmd), inet_ntoa (p->u.prefix4),
-                         p->prefixlen, inet_ntoa (nexthop->rgate.ipv4),
-                         nexthop->rifindex, 
-                         nexthop_types_desc[nexthop->type]);
+                      zlog_debug ("netlink_route_multipath() (multihop): "
+                         "%s %s/%d, type %s", lookup (nlmsg_str, cmd),
+#ifdef HAVE_IPV6
+			 (family == AF_INET) ? inet_ntoa (p->u.prefix4) :
+			 inet6_ntoa (p->u.prefix6),
+#else
+			 inet_ntoa (p->u.prefix4),
+#endif /* HAVE_IPV6 */
+			 p->prefixlen, nexthop_types_desc[nexthop->type]);
                     }
                   if (nexthop->type == NEXTHOP_TYPE_IPV4
                       || nexthop->type == NEXTHOP_TYPE_IPV4_IFINDEX)
                     {
-                      rta_addattr_l (rta, 4096, RTA_GATEWAY,
-                                     &nexthop->gate.ipv4, bytelen);
-                      rtnh->rtnh_len += sizeof (struct rtattr) + 4;
+		      rta_addattr_l (rta, 4096, RTA_GATEWAY,
+				     &nexthop->gate.ipv4, bytelen);
+		      rtnh->rtnh_len += sizeof (struct rtattr) + 4;
+
+                      if (IS_ZEBRA_DEBUG_KERNEL)
+			zlog_debug("netlink_route_multipath() (multihop): "
+				   "nexthop via %s if %u",
+				   inet_ntoa (nexthop->gate.ipv4),
+				   nexthop->ifindex);
                     }
 #ifdef HAVE_IPV6
                   if (nexthop->type == NEXTHOP_TYPE_IPV6
                       || nexthop->type == NEXTHOP_TYPE_IPV6_IFNAME
                       || nexthop->type == NEXTHOP_TYPE_IPV6_IFINDEX)
-                    rta_addattr_l (rta, 4096, RTA_GATEWAY,
-                                   &nexthop->gate.ipv6, bytelen);
+		    { 
+		      rta_addattr_l (rta, 4096, RTA_GATEWAY,
+				     &nexthop->gate.ipv6, bytelen);
+
+		      if (IS_ZEBRA_DEBUG_KERNEL)
+			zlog_debug("netlink_route_multipath() (multihop): "
+				   "nexthop via %s if %u",
+				   inet6_ntoa (nexthop->gate.ipv6),
+				   nexthop->ifindex);
+		    }
 #endif /* HAVE_IPV6 */
                   /* ifindex */
                   if (nexthop->type == NEXTHOP_TYPE_IFINDEX
@@ -1589,9 +1681,17 @@ netlink_route_multipath (int cmd, struct prefix *p, struct rib *rib,
                       || nexthop->type == NEXTHOP_TYPE_IPV4_IFINDEX
                       || nexthop->type == NEXTHOP_TYPE_IPV6_IFNAME
                       || nexthop->type == NEXTHOP_TYPE_IPV6_IFINDEX)
-                    rtnh->rtnh_ifindex = nexthop->ifindex;
+		    {
+		      rtnh->rtnh_ifindex = nexthop->ifindex;
+
+		      if (IS_ZEBRA_DEBUG_KERNEL)
+			zlog_debug("netlink_route_multipath() (multihop): "
+				   "nexthop via if %u", nexthop->ifindex);
+		    }
                   else
-                    rtnh->rtnh_ifindex = 0;
+		    {
+		      rtnh->rtnh_ifindex = 0;
+		    }
                 }
               rtnh = RTNH_NEXT (rtnh);
 
